//===== Copyright  1996-2008, Valve Corporation, All rights reserved. ======//
//
// Purpose: 
//
// $NoKeywords: $
//
//===========================================================================//
#include "winlite.h"
#include "cbase.h"
#include <stdio.h>
#include "interface.h"
#include "filesystem.h"
#include "engine/iserverplugin.h"
#include "eiface.h"
#include "igameevents.h"
#include "convar.h"
#include "Color.h"
#include "vstdlib/random.h"
#include "engine/IEngineTrace.h"
#include "tier2/tier2.h"

#include "CGmodBasePlugin.h"

#include "detours.h"
#include "SigScan.h"

LUA_FUNCTION(DumpInfo)
{
	ILuaInterface *g = gMod->m_lua;

	g->CheckType(1, GLua::TYPE_STRING);

	INetworkStringTable *netTbl = networkstringtable->FindTable(g->GetString(1));

	if(netTbl)
	{
		g->Msg("%d %s %d %d %d\n", netTbl->a, netTbl->b, netTbl->c, netTbl->d);
	}

	return 0;
}

LUA_FUNCTION(SetTableSize)
{
	ILuaInterface *g = gMod->m_lua;

	g->CheckType(1, GLua::TYPE_STRING);
	g->CheckType(2, GLua::TYPE_NUMBER);

	INetworkStringTable *netTbl = networkstringtable->FindTable(g->GetString(1));

	if(netTbl)
	{
		netTbl->c = g->GetInteger(2);
		netTbl->d = Q_log2(netTbl->c);
	}

	return 0;
}

LUA_FUNCTION(GetTableNames)
{
	ILuaInterface *g = gMod->m_lua;

	ILuaObject *tbl = g->GetNewTable();

	for(int i = 0; i < networkstringtable->GetNumTables(); i++)
	{
		tbl->SetMember(i, networkstringtable->GetTable(i)->GetTableName());
	}

	g->Push(tbl);

	tbl->UnReference();

	return 1;
}

LUA_FUNCTION(GetNTable)
{
	ILuaInterface *g = gMod->m_lua;

	g->CheckType(1, GLua::TYPE_STRING);

	INetworkStringTable *netTbl = networkstringtable->FindTable(g->GetString(1));

	if(netTbl)
	{
		ILuaObject *tbl = g->GetNewTable();

		for(int i = 0; i < netTbl->GetNumStrings(); i++)
		{
			int b;
			tbl->SetMember(netTbl->GetString(i), (const char*)netTbl->GetStringUserData(i, &b));
		}

		g->Push(tbl);

		tbl->UnReference();
		return 1;
	}

	return 0;
}

LUA_FUNCTION(SetTableMember)
{
	ILuaInterface *g = gMod->m_lua;

	g->CheckType(1, GLua::TYPE_STRING);
	g->CheckType(2, GLua::TYPE_STRING);
	g->CheckType(3, GLua::TYPE_STRING);

	INetworkStringTable *netTbl = networkstringtable->FindTable(g->GetString(1));

	if(netTbl)
	{
		int idx = netTbl->FindStringIndex(g->GetString(2));
		if(idx == INVALID_STRING_INDEX)
		{
			idx = netTbl->AddString(g->IsServer(),g->GetString(2));
		}

		netTbl->SetStringUserData(idx,strlen(g->GetString(3)), g->GetString(3));
	}

	return 0;
}

void CGmodBasePlugin::Init()
{
	CSigScan::sigscan_dllfunc = Sys_GetFactory("server.dll");
	bool success = CSigScan::GetDllMemInfo();

	if(!success)
	{
		m_lua->Msg("Found Mem Info @ 0x%x\n", (size_t)CSigScan::base_addr);
	}

	ILuaObject *tbl = m_lua->GetNewTable();

	tbl->SetMember("GetTableNames", GetTableNames);
	tbl->SetMember("GetTable", GetNTable);
	tbl->SetMember("SetTableMember", SetTableMember);
	tbl->SetMember("DumpInfo",DumpInfo);
	tbl->SetMember("SetTableSize",SetTableSize);
	
	m_lua->SetGlobal("NetTbl", tbl);
	tbl->UnReference();
}

void CGmodBasePlugin::Shutdown()
{
}